@@ -2,6 +2,7 @@ package Form::Processor::Field::DateTimeDMYHM2;
 use strict;
 use warnings;
 use base 'Form::Processor::Field';
+use Form::Processor;
 use DateTime;
 
 # This implements a field made up of sub fields.
@@ -9,15 +10,12 @@ use DateTime;
 sub init_widget { 'Compound' }
 
 
-use Rose::Object::MakeMethods::Generic (
-    scalar => [
-        sub_form => {},
-    ],
-);
-
-
 our $VERSION = '0.01';
 
+# This is to keep from reporting missing field
+# for required fields.  Any missing data errors will propogate up.
+sub any_input { 1 }
+
 sub init {
     my $self = shift;
 
@@ -25,9 +23,24 @@ sub init {
 
     my $name = $self->name;
 
+    my $required = $self->required ? 'required' : 'optional';
+
     $self->sub_form(
-         Form::Processor::Field::DateTimeDMYHM2::Form->new( name_prefix => $name )
+        Form::Processor->new(
+            parent_field => $self,  # send all errors to parent field.
+            profile => {
+                optional=> {
+                    day     => 'MonthDay',
+                    month   => 'MonthName',
+                    year    => 'Year',
+                    hour    => 'Hour',
+                    minute  => 'Minute',
+                },
+            },
+        )
     );
+
+    return;
 }
 
 
@@ -38,7 +51,7 @@ sub validate {
     my $form = $self->sub_form;
 
     # First validate the sub fields, passing in the original parameters
-    return unless $sub_form->validate( $self->form->params );
+    return unless $form->validate( scalar $self->form->params );
 
     my %date = map { $_ => $form->field($_)->value } qw/ day month year hour minute /;
 
@@ -46,13 +59,15 @@ sub validate {
     eval {  $dt = DateTime->new( %date, time_zone => 'floating' ) };
 
     if ( $@ ) {
-        $self->add_error( "Invalid date [_1]", "$@" );
+        my $error = $@;
+        $error =~ s! at /.+$!!; # ! vim
+        $self->add_error( "Invalid date [_1]", "$error" );
         return;
     }
 
     $self->value( $dt );
 
-    1;
+    return 1;
 }
 
 sub format_value {
@@ -78,38 +93,4 @@ sub format_value {
 1;
 
 
-package Form::Processor::Field::DateTimeDMYHM2::Form;
-use strict;
-use warnings;
-use base 'Form::Processor::Base';
-
-sub profile {
-    return {
-        auto_required => [ qw/ day month year hour minute /],
-    };
-}
-
-sub guess_field_type { 'Select' }
-
-sub options_day { [ map { $_, $_ } 1..31 ] }
-sub options_year { [ map { $_, $_ } 2000..2010 ] }
-sub options_hour { [ map { $_, $_ } 1..23 ] }
-sub options_minute { [ map { $_, $_ } 0..59 ] }
-
-
-# I18N
-warn "i18n";
-sub options_month { 
-    my $self = shift;
-    my $n = 1;
-    my @m = qw/
-        January Feburary March April May June July
-        August September October November December
-    /;
-    return [ map { $n++, $_ } @m  ];
-}
-
-
-
-1;
 
