@@ -3,7 +3,7 @@ use strict;
 use warnings;
 use base 'Form::Processor::Field';
 
-sub init_widget { 'Text' }
+sub init_widget { 'text' }
 
 
 our $VERSION = '0.01';
